require 'ci_helper/version'
module CiHelper
  module Bitbucket

    class PullRequestPage

      # This class is responsible for the update the pull request form in the bitbucket
      # example :
      # object.form_set do |config|
      #   config.title = 'your title'
      #   config.destination = 'the branch name you want to merge'
      #   config.reviewer = 'the reviewer you want to add'
      #   config.append_description = 'text '
      #   config.description_syntax('ruby') do |content|
      #     content.value += 'your code'
      #   end
      # end

      attr_accessor :form, :point_counter

      def initialize
        @point_counter = 1
      end

      def title=(text)
        form.field_with(name: 'title').value = text
      end

      def destination=(dest)
        form.field_with(name: 'dest').options.select { |option| option.text == dest.to_s}.first.select
      end

      def source=(branch_name)
        form.field_with(name: 'source').options.select { |option| option.text == branch_name}.first.select
      end

      def reviewers=(reviewers)
        form.field_with(name: 'reviewers').value = reviewers
      end

      def description_point=(text)
        description.value += "#{point_counter}. #{text}\n"
        self.point_counter += 1
      end

      def append_description=(append_text)
        description.value += "\n#{append_text}\n"
      end

      def description_syntax(language, &failure_block)
        description.value += "```\n"
        description.value += "#!#{language} \n"
        failure_block.call(description) if block_given?
        description.value += "```\n"
      end

      def submit
        description.value += "\n CiHelper Version #{::CiHelper::VERSION}"
        form.click_button
      end

      private

      def description
        form.field_with(name: 'description')
      end
    end
  end
end