require 'mechanize'

module CiHelper
  class BitBucketProcedure
    BIT_BUCKET_LOGIN_URL = 'https://bitbucket.org/account/signin/'
    include CiHelper::Bitbucket

    attr_reader :account, :password, :client, :destination, :user_description, :title, :reviewers
    attr_reader :pull_request_page, :repository_page, :branch_name, :result

    def initialize(account, password)
      @account  = account
      @password = password
      @client   = Mechanize.new
    end

    def create_pull_request
      login
      go_to_repo_page
      go_to_pull_request_page

      create_pull_request_page = pull_request_page.link_with(href: '/vansys/vansys/pull-request/new').click
      set_form(create_pull_request_page.form_with(action: '/vansys/vansys/pull-request/new'))

      form_set do |config|
        config.title       = "#{issue_numbers.join(' ')} - #{title}"
        config.destination = destination.to_s
        config.source      = branch_name
        config.reviewers   = reviewers

        issue_numbers.each do |number|
          config.description_point = "#{CiHelper::REDMINE_DOMAIN}#{number}"
        end

        if result
          config.description_syntax('ruby') do |content|
            result.failures.each do |failure|
              content.append_description = failure.to_s
            end
          end
        end

        config.append_description = user_description
      end

      submit
    end

    def add_destination(destination)
      @destination = destination
    end

    def add_user_description(description)
      @user_description = description
    end

    def pf_title=(title)
      @title = title
    end

    def multiple_issues?
      issue_numbers.size > 1
    end

    # transform the user input branch name t3001 t3001_reopen or t3001_t3002_t3003 to [3001], [3001] or [3001,3002,3003]
    def issue_numbers
      branch_name.split('_').each { |name| name.delete!('t') }.delete_if { |name| name.to_i.zero?}
    end

    def branch_name=(name)
      @branch_name = name
    end

    def reviewers=(reviewers)
      @reviewers = reviewers
    end

    def result=(result)
      @result = result
    end

    def login
      login_page = client.get(BIT_BUCKET_LOGIN_URL)

      @response_page = login_page.form_with(action: '/account/signin/') do |f|
        f.field_with(name: 'username').value = account
        f.field_with(name: 'password').value = password
      end.click_button
    end

    def go_to_repo_page
      fail Exceptions::SettingError, "Please make sure you enter the correct account and password \n and run ci_helper -i again" if login_failed?
      @repository_page = @response_page.link_with(href: '/vansys/vansys').click
    end

    def go_to_pull_request_page
      @pull_request_page = repository_page.link_with(href: '/vansys/vansys/pull-requests').click
    end

    private

    def login_failed?
      @response_page.uri.request_uri.match('signin')
    end
  end
end
