require 'optparse'

module CiHelper
  # Responsible for parsing command-line option and executing appropriate
  # application logic based on those options
  class CLI


    attr_reader :arguments, :input, :options, :parser

    def initialize(arguments, input)
      @arguments       = arguments
      @input           = input
      @options         = {}
      options[:target] = CiHelper::Utils.repo_root
    end

    def run
      @parser = create_option_parser
      parser.parse!(arguments)
      if initialize?
        user_setting = get_initialize_data
        Installer.new(options[:target], user_setting).install_ci_helper_config
      elsif run?
        MainProcess.new(options[:target]).run
      else
        puts parser.help
      end
    end

    private

    def create_option_parser
      OptionParser.new do |opts|
        opts.banner = "Usage: #{opts.program_name} [options]"

        add_information_options(opts)
      end
    end

    def add_information_options(opts)
      opts.on('-h', '--help', 'Show this message') do
        puts opts.help
      end

      opts.on('-i', '--install', 'Install a config setting by user input') do
        @options[:action] = :install
      end

      opts.on('-r', '--run', 'run the ci helper') do
        @options[:action] = :run
      end
    end

    def initialize?
      options[:action] == :install
    end

    def run?
      options[:action] == :run
    end

    def get_initialize_data
      puts 'Please input your ci server account '
      ci_account = gets.chop
      puts 'Please input your ci server password'
      ci_password = gets.chop
      puts 'Please input your bitbucket account'
      bitbucket_account = gets.chop
      puts 'Please input your bitbucket password'
      bitbucket_password = gets.chop
      puts 'Please input your redmine account'
      redmine_ac = gets.chop
      puts 'Please input your remind password'
      redmine_pass = gets.chop

      { ci:        { account: ci_account, password: ci_password },
        bitbucket: { account: bitbucket_account, password: bitbucket_password },
        redmine: { account: redmine_ac, password: redmine_pass}}
    end
  end
end
