module CiHelper
  class IssueMonitor
    include CiHelper::RspecResultParser

    attr_accessor :issue_status
    attr_reader :ci_procedure

    Result = Struct.new(:status, :failures,
                        :console_link, :issue_number,
                        :branch_name, :commit_hash)

    def initialize(account, password)
      @ci_procedure = CiProcedure.new(account, password)
    end

    def matching_strategy=(pattern)
      ci_procedure.matching_strategy = pattern
    end

    def matching_strategy
      ci_procedure.matching_strategy
    end

    def goto_monitor_page
      begin
        ci_procedure.run
      rescue CiHelper::Exceptions::IssueNotFound => e
        puts "#{e.message} #{ci_procedure.matching_strategy}"
        return false
      end

      true
    end

    def success?
      parser.console_output = ci_procedure.get_console_output
      parser.formalize_output
      parser.success?
    end

    def finish_test?
      begin
        ci_procedure.re_get_console_output.empty? ? false : true
      rescue CiHelper::Exceptions::IssueNotFound => e
        puts "#{e.message} #{ci_procedure.matching_strategy}"
        false
      end
    end

    def failures_each
      parser.fail_examples.each { |failed_example| yield(failed_example) }
    end

    def content
      Result.new(success?, parser.fail_examples,
                 "#{ci_procedure.console_page.uri.host}/#{ci_procedure.console_page.uri.request_uri}",
                 ci_procedure.issue_number,
                 ci_procedure.branch_name,
                 ci_procedure.commit_hash)
    end
  end
end
